﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinListView;

namespace RearrangingListViewItems
{
    public partial class Form1 : Form
    {
        private Nullable<Point> lastMouseDown;
        private UltraListViewItem dragItem;
        private UltraListViewItem dropItem;
        public Form1()
        {
            InitializeComponent();
            this.StartPosition = FormStartPosition.CenterScreen;
            this.listView.MouseDown += new MouseEventHandler(listView_MouseDown);
            this.listView.MouseMove += new MouseEventHandler(listView_MouseMove);
            this.listView.DragOver += new DragEventHandler(listView_DragOver);
            this.listView.DragDrop += new DragEventHandler(listView_DragDrop);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            lastMouseDown = null;
            dragItem = null;
            dropItem = null;

            listView.AllowDrop = true;
           
            this.listView.QueryContinueDrag += new QueryContinueDragEventHandler(listView_QueryContinueDrag);

            this.listView.View = Infragistics.Win.UltraWinListView.UltraListViewStyle.Thumbnails;
            this.listView.ViewSettingsTiles.ImageList = this.imageList1;
            this.listView.ViewSettingsThumbnails.ImageList = this.imageList1;
            this.listView.ViewSettingsTiles.ItemSize = new Size(60, 60);
            this.listView.ViewSettingsThumbnails.ItemSize = new Size(60, 60);
            for (int i = 0; i < this.imageList1.Images.Count; i++)
            {
                this.listView.Items[i].Appearance.Image = i;
                this.listView.Items[i].Key = this.imageList1.Images.Keys[i];
            }

        }

        private void listView_ItemActivated(object sender, ItemActivatedEventArgs e)
        {
            if (e.Item != null)
            {
                this.ultraPictureBox1.Image = this.imageList1.Images[e.Item.Key];
            }
        }

        private void listView_MouseDown(object sender, MouseEventArgs e)
        {
            //  Record the cursor location
            if (e.Button == MouseButtons.Left)
            {
                this.lastMouseDown = e.Location;
            }
            else
               {
                   if (this.listView.ActiveItem != null)
                   {
                       this.listView.Items.Remove(this.listView.ActiveItem);
                       this.lastMouseDown = null;
                   }
                   else
                   {
                       this.lastMouseDown = null;
                   }
               }
        }

        private void listView_MouseMove(object sender, MouseEventArgs e)
        {
            UltraListView listView = sender as UltraListView;


            //  If the mouse has moved outside the area in which it was pressed,
            //  start a drag operation
            if (this.lastMouseDown.HasValue)
            {
                Size dragSize = SystemInformation.DragSize;
                Rectangle dragRect = new Rectangle(this.lastMouseDown.Value, dragSize);
                dragRect.X -= dragSize.Width / 2;
                dragRect.Y -= dragSize.Height / 2;

                if (!dragRect.Contains(e.Location))
                {
                    UltraListViewItem itemAtPoint = listView.ItemFromPoint(e.Location);

                    if (itemAtPoint != null)
                    {
                        this.lastMouseDown = null;
                        this.dragItem = itemAtPoint;
                        listView.DoDragDrop(this.dragItem, DragDropEffects.Move);
                    }
                }
            }
        }

        private void listView_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            UltraListView listView = sender as UltraListView;

            //  Cancel the drag operation if the escape key was pressed
            if (e.EscapePressed)
            {
                this.OnDragEnd(listView, true);
                e.Action = DragAction.Cancel;
            }
        }

        private void listView_DragOver(object sender, DragEventArgs e)
        {
            UltraListView listView = sender as UltraListView;
            Point clientPos = listView.PointToClient(new Point(e.X, e.Y));

            if (this.dragItem != null)
            {
                this.dropItem = listView.ItemFromPoint(clientPos);

                e.Effect = this.dropItem != null && this.dropItem != this.dragItem ? DragDropEffects.Move : DragDropEffects.None;
            }

            //  TODO: This could be improved with a hover timer

            //  If the cursor is within {dragScrollAreaHeight} pixels
            //  of the top or bottom edges of the control, scroll
            int dragScrollAreaHeight = 8;

            Rectangle displayRect = listView.DisplayRectangle;
            Rectangle topScrollArea = displayRect;
            topScrollArea.Height = (dragScrollAreaHeight * 2);

            Rectangle bottomScrollArea = displayRect;
            bottomScrollArea.Y = bottomScrollArea.Bottom - dragScrollAreaHeight;
            bottomScrollArea.Height = dragScrollAreaHeight;

            ISelectionManager selectionManager = listView as ISelectionManager;
            if (topScrollArea.Contains(clientPos) || bottomScrollArea.Contains(clientPos))
                selectionManager.DoDragScrollVertical(0);
        }

        private void listView_DragDrop(object sender, DragEventArgs e)
        {
            UltraListView listView = sender as UltraListView;
            this.OnDragEnd(listView, false);
        }

        private void OnDragEnd(UltraListView listView, bool canceled)
        {
            if (canceled == false && this.dragItem != null && this.dropItem != null)
            {
                listView.BeginUpdate();
                //int index = this.dropItem.Index;


                listView.Items.Remove(this.dragItem);
                int index = this.dropItem.Index + 1;
                listView.Items.Insert(index, this.dragItem);

                listView.EndUpdate();
            }

            this.dragItem = this.dropItem = null;
            this.lastMouseDown = null;
        }
    }
}
